﻿using OrganizationDiagramSample.Model;
using OrganizationDiagramSample.ViewModel;
using Syncfusion.UI.Xaml.Diagram;
using Syncfusion.UI.Xaml.Diagram.Layout;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;
using Windows.Foundation;
using Windows.UI;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Animation;

namespace OrganizationDiagramSample.Utilities
{
    public class ComboBoxCommand
    {
        public static readonly DependencyProperty CommandProperty = DependencyProperty.RegisterAttached("Command", typeof(ICommand),
        typeof(ComboBoxCommand), new PropertyMetadata(null, PropertyChangedCallback));

        public static void PropertyChangedCallback(DependencyObject depObj, DependencyPropertyChangedEventArgs args)
        {
            ComboBox comboBox = depObj as ComboBox;
            if (comboBox != null)
            {
                //ICommand command = comboBox.GetValue(CommandProperty) as ICommand;
                //command.Execute(comboBox);
                comboBox.SelectionChanged += comboBox_SelectionChanged;
            }
        }

        static void comboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ComboBox comboBox = sender as ComboBox;
            if (comboBox != null)
            {

                ICommand command = comboBox.GetValue(CommandProperty) as ICommand;
                if (command != null)
                {
                    command.Execute(comboBox);
                }
            }
        }
        public static ICommand GetCommand(UIElement element)
        {
            return (ICommand)element.GetValue(CommandProperty);
        }

        public static void SetCommand(UIElement element, ICommand command)
        {
            element.SetValue(CommandProperty, command);
        }
    }

    public class TextBoxCommand
    {
        public static readonly DependencyProperty CommandProperty = DependencyProperty.RegisterAttached("Command", typeof(ICommand),
        typeof(TextBoxCommand), new PropertyMetadata(null, PropertyChangedCallback));

        public static void PropertyChangedCallback(DependencyObject depObj, DependencyPropertyChangedEventArgs args)
        {
            TextBox textBox = depObj as TextBox;
            if (textBox != null)
            {
                textBox.TextChanged += textBox_TextChanged;
            }
        }

        static void textBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            TextBox textBox = (sender as TextBox);
            if (textBox != null)
            {
                textBox.Foreground = new SolidColorBrush(Color.FromArgb(0x33, 0x33, 0x33, 0));

                ICommand command = textBox.GetValue(CommandProperty) as ICommand;
                if (command != null)
                {
                    command.Execute(textBox.Text);
                }
            }
        }

        public static ICommand GetCommand(UIElement element)
        {
            return (ICommand)element.GetValue(CommandProperty);
        }

        public static void SetCommand(UIElement element, ICommand command)
        {
            element.SetValue(CommandProperty, command);
        }
    }

    public class DiagramCommand
    {
        public static readonly DependencyProperty CommandProperty = DependencyProperty.RegisterAttached("Command", typeof(ICommand),
        typeof(DiagramCommand), new PropertyMetadata(null, PropertyChangedCallback));

        private static OrgChartDiagram diagram = null;
        public static void PropertyChangedCallback(DependencyObject depObj, DependencyPropertyChangedEventArgs args)
        {
            diagram = depObj as OrgChartDiagram;
            if (args.NewValue == null)
            {
                (diagram.Info as IGraphInfo).ItemTappedEvent -= diagram_ItemTappedEvent;
                diagram.Loaded -= sfdiagram_Loaded;
                diagram = null;
            }
            if (diagram != null)
            {
                (diagram.Info as IGraphInfo).ItemTappedEvent += diagram_ItemTappedEvent;
                diagram.Loaded += sfdiagram_Loaded;
            }
        }

        static void diagram_ItemTappedEvent(object sender, DiagramEventArgs args)
        {
        }

        static void sfdiagram_Loaded(object sender, RoutedEventArgs e)
        {
            foreach (var item in (sender as SfDiagram).Page.Children.OfType<Node>())
            {
                if (item.GetType() != typeof(Syncfusion.UI.Xaml.Diagram.Selector))
                {
                    if (((item as Node).DataContext as OrgNodeViewModel).Content is UserInfo)
                    {
                        //(item as Node).IsExpanded = false;
                        if ((((item as Node).DataContext as OrgNodeViewModel).Content as UserInfo).IsExpand == State.Collapse)
                        {
                            (((item as Node).DataContext as OrgNodeViewModel).Content as UserInfo).IsExpand = State.Collapse;
                            //((item as Node).DataContext as OrgNodeViewModel).IsExpanded = false;
                            Hide(((item as Node).DataContext as OrgNodeViewModel));
                            //(sender as OrgChartDiagram).LayoutManager.Layout.UpdateLayout(((item as Node).DataContext as OrgNodeViewModel));                          
                        }
                    }
                    else
                    {
                        (item as Node).IsHitTestVisible = false;
                    }
                }
            }
        }


        //Collapses the Child Nodes
        private static void Hide(OrgNodeViewModel n)
        {
            foreach (OrgNodeViewModel node in diagram.Page.Children.OfType<Connector>()
                                  .Where(connector => (connector.SourceNode as OrgNodeViewModel) == n)
                                  .Select(connector => (connector.TargetNode as OrgNodeViewModel)))
            {
                foreach (Connector line in diagram.Page.Children.OfType<Connector>()
                                  .Where(connector => connector.TargetNode == node || connector.SourceNode == node))
                {

                    //line.Visibility = Visibility.Collapsed;
                    (line.DataContext as OrgConnectorViewModel).Visible = false;
                    CollapseAnimation(line);
                }
                foreach (Node v in diagram.Page.Children.OfType<Node>())
                {
                    if (v.DataContext == node)
                    {
                        CollapseAnimation(v);
                        //v.Visibility = Visibility.Collapsed;
                        (v.DataContext as OrgNodeViewModel).Visible = false;
                    }

                }
                Hide(node);
            }
        }



        //Fade out animation
        static void CollapseAnimation(DependencyObject obj)
        {
            Duration duration = new Duration(TimeSpan.FromMilliseconds(300));
            // Create two DoubleAnimations and set their properties.
            DoubleAnimation myDoubleAnimation1 = new DoubleAnimation();
            myDoubleAnimation1.Duration = duration;
            Storyboard sb = new Storyboard();
            sb.Duration = duration;
            sb.Children.Add(myDoubleAnimation1);
            Storyboard.SetTarget(myDoubleAnimation1, obj);
            Storyboard.SetTargetProperty(myDoubleAnimation1, "Opacity");
            myDoubleAnimation1.From = 1d;
            myDoubleAnimation1.To = 0d;
            // Begin the animation.
            sb.Begin();
        }
        static void textBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            TextBox textBox = (sender as TextBox);
            if (textBox != null)
            {
                ICommand command = textBox.GetValue(CommandProperty) as ICommand;
                if (command != null)
                {
                    command.Execute(textBox.Text);
                }
            }
        }

        public static ICommand GetCommand(UIElement element)
        {
            return (ICommand)element.GetValue(CommandProperty);
        }

        public static void SetCommand(UIElement element, ICommand command)
        {
            element.SetValue(CommandProperty, command);
        }
    }

    public class PathCommand
    {
        public static readonly DependencyProperty CommandProperty = DependencyProperty.RegisterAttached("Command", typeof(ICommand),
        typeof(PathCommand), new PropertyMetadata(null, PropertyChangedCallback));

        public static void PropertyChangedCallback(DependencyObject depObj, DependencyPropertyChangedEventArgs args)
        {
            UIElement p = (depObj as UIElement);
            if (p != null)
            {
                p.Tapped += p_Tapped;
            }

        }

        static void p_Tapped(object sender, Windows.UI.Xaml.Input.TappedRoutedEventArgs e)
        {

            UIElement button = sender as UIElement;
            if (button != null)
            {
                ICommand command = button.GetValue(CommandProperty) as ICommand;
                if (command != null)
                {
                    command.Execute(button);
                }
            }
            e.Handled = true;
        }

        public static ICommand GetCommand(UIElement element)
        {
            return (ICommand)element.GetValue(CommandProperty);
        }

        public static void SetCommand(UIElement element, ICommand command)
        {
            element.SetValue(CommandProperty, command);
        }
    }

    public class ButtonCommand
    {
        public static readonly DependencyProperty CommandProperty = DependencyProperty.RegisterAttached("Command", typeof(ICommand),
        typeof(ButtonCommand), new PropertyMetadata(null, PropertyChangedCallback));

        public static void PropertyChangedCallback(DependencyObject depObj, DependencyPropertyChangedEventArgs args)
        {
            Grid button = depObj as Grid;
            if (button != null)
            {
                button.Tapped += button_Tapped;
            }
        }

        static void button_Tapped(object sender, Windows.UI.Xaml.Input.TappedRoutedEventArgs e)
        {
            Grid button = sender as Grid;
            if (button != null)
            {
                ICommand command = button.GetValue(CommandProperty) as ICommand;
                if (command != null)
                {
                    command.Execute(button.Name);
                }
            }
        }

        static void button_Click(object sender, RoutedEventArgs e)
        {

        }

        public static ICommand GetCommand(UIElement element)
        {
            return (ICommand)element.GetValue(CommandProperty);
        }

        public static void SetCommand(UIElement element, ICommand command)
        {
            element.SetValue(CommandProperty, command);
        }
    }

    public static class FrameworkElementExtensions
    {
        public static FrameworkElement FindDescendantByName(this FrameworkElement element, string name)
        {
            if (element == null || string.IsNullOrWhiteSpace(name)) { return null; }

            if (name.Equals(element.Name, StringComparison.OrdinalIgnoreCase))
            {
                return element;
            }
            var childCount = VisualTreeHelper.GetChildrenCount(element);
            for (int i = 0; i < childCount; i++)
            {
                var result = (VisualTreeHelper.GetChild(element, i) as FrameworkElement).FindDescendantByName(name);
                if (result != null) { return result; }
            }
            return null;
        }
    }
}
